//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include "preferences.h"
#include <time.h>
#include "opcodes.h"
#include "otherfunctions.h"
#include "ini2.h"
#include "stdlib.h"
#include "stdio.h"
#include <locale.h>
#include "resource.h"
#include "emule.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define ELEMENT_COUNT(X) (sizeof(X) / sizeof(X[0]))

CPreferences::CPreferences(){
	srand((uint32)time(0)); // we need random numbers sometimes

	prefs = new Preferences_Struct_DONTUSE;
//	memset(prefs,0,sizeof(Preferences_Struct));
	prefsExt=new Preferences_Ext_Struct;
	memset(prefsExt,0,sizeof(Preferences_Ext_Struct));

	//get application start directory
	char buffer[490];
	::GetModuleFileName(0, buffer, 490);
	LPTSTR pszFileName = _tcsrchr(buffer, '\\') + 1;
	*pszFileName = '\0';
	appdir = nstrdup(buffer);
	CreateUserHash();
	memcpy(&prefs->userhash,&userhash,16);

	// load preferences.dat or set standart values
	char* fullpath = new char[strlen(appdir)+MAX_PATH]; // i_a
	sprintf(fullpath,"%spreferences.dat",appdir);
	FILE* preffile = fopen(fullpath,"rb");
	delete[] fullpath;

	LoadPreferences();
	if (!preffile){
		SetStandartValues();
		//if (Ask4RegFix(true)) Ask4RegFix(false);
	}
	else{
		fread(prefsExt,sizeof(Preferences_Ext_Struct),1,preffile);
		if (ferror(preffile))
			SetStandartValues();

		// import old pref-files
		if (prefsExt->version<20) {


			if (prefsExt->version>17) {// v0.20b+
				prefsImport20b=new Preferences_Import20b_Struct;
				memset(prefsImport20b,0,sizeof(Preferences_Import20b_Struct));
				fseek(preffile,0,0);
				fread(prefsImport20b,sizeof(Preferences_Import20b_Struct),1,preffile);

				memcpy(&prefs->userhash,&prefsImport20b->userhash,16);
				m_sIncomingDir.Put(prefsImport20b->incomingdir);
				m_sTempDir.Put(prefsImport20b->tempdir);
				m_sNick.Put(prefsImport20b->nick);

				m_nTotalDownloadedBytes.Put(prefsImport20b->totalDownloadedBytes);
				m_nTotalUploadedBytes.Put(prefsImport20b->totalUploadedBytes);

			} else if (prefsExt->version>7) { // v0.20a
				prefsImport20a=new Preferences_Import20a_Struct;
				memset(prefsImport20a,0,sizeof(Preferences_Import20a_Struct));
				fseek(preffile,0,0);
				fread(prefsImport20a,sizeof(Preferences_Import20a_Struct),1,preffile);

				memcpy(&prefs->userhash,&prefsImport20a->userhash,16);
				m_sIncomingDir.Put(prefsImport20a->incomingdir);
				m_sTempDir.Put(prefsImport20a->tempdir);
				m_sNick.Put(prefsImport20a->nick);

				m_nTotalDownloadedBytes.Put(prefsImport20a->totalDownloaded);
				m_nTotalUploadedBytes.Put(prefsImport20a->totalUploaded);

			} else {	//v0.19c-
				prefsImport19c=new Preferences_Import19c_Struct;
				memset(prefsImport19c,0,sizeof(Preferences_Import19c_Struct));

				fseek(preffile,0,0);
				fread(prefsImport19c,sizeof(Preferences_Import19c_Struct),1,preffile);

				if (prefsExt->version<3) {
					CreateUserHash();
					memcpy(&prefs->userhash,&userhash,16);
				} else {memcpy(&prefs->userhash,&prefsImport19c->userhash,16);}
				m_sIncomingDir.Put(prefsImport19c->incomingdir);
				m_sTempDir.Put(prefsImport19c->tempdir);
				m_sNick.Put(prefsImport19c->nick);
			}
 		} else {
			memcpy(&prefs->userhash,&prefsExt->userhash,16);
			prefs->EmuleWindowPlacement=prefsExt->EmuleWindowPlacement;
		}
		fclose(preffile);
	memcpy(&userhash,&prefs->userhash,16);
	prefs->smartidstate=0;
	}

	// shared directories
	fullpath = new char[strlen(appdir)+MAX_PATH]; // i_a
	sprintf(fullpath,"%sshareddir.dat",appdir);
	CStdioFile* sdirfile = new CStdioFile();
	if(sdirfile->Open(fullpath,CFile::modeRead))
	{
		CString toadd;
		while (sdirfile->ReadString(toadd))
			shareddir_list.AddHead(toadd);
		sdirfile->Close();
	}
	delete sdirfile;

	// start added by InterCeptor (multiple tempdirs) 19.11.02
	// temp directories
	//fullpath = new char[strlen(appdir)+MAX_PATH]; [FoRcHa] (we already have one)
	sprintf(fullpath,"%stempdir.dat",appdir);
	CStdioFile* tdirfile = new CStdioFile();
	if (tdirfile->Open(fullpath,CFile::modeRead)){
		CString toadd;
		while (tdirfile->ReadString(toadd)){
			tempdir_list.AddHead(toadd);
			::CreateDirectory(toadd,0);
		}
		tdirfile->Close();
	}
	delete tdirfile;
	// end added by InterCeptor (multiple tempdirs) 19.11.02

	delete[] fullpath;
	
	//serverlist adresses
	LoadServerlistAddresses() ;

	userhash[5] = 14;
	userhash[14] = 111;

	//Maverick
    m_UseProxyListenPort = false;
	ListenPort = 0;

	CreateDirectory(GetIncomingDir(),0);
	CreateDirectory(GetTempDir(),0);
	//CreateDirectory(GetVlcDir(),0);		//Cax2 not needed anymore
}

// Added by Tarod
void CPreferences::LoadServerlistAddresses()
{
	// First we must free list
	adresses_list.RemoveAll() ;

	CString toadd ;
	CString fullpath = CString(appdir) + "adresses.dat" ;
	CStdioFile* sdirfile = new CStdioFile() ;
	if (sdirfile->Open(fullpath, CFile::modeRead)) {
		adresses_list.RemoveAll() ;
		while (sdirfile->ReadString(toadd)) {
        	// [i_a] allow commented-out entries in the addresses.dat file.
                toadd.TrimLeft();
                if (toadd.GetLength() > 0 && toadd[0] != '#')
			adresses_list.AddHead(toadd) ;
		}
		sdirfile->Close() ;
	}
	delete sdirfile ;
}

void CPreferences::SetStandartValues(){
	// CreateUserHash();
	// Saving HashID 
	char* fullpath = new char[strlen(appdir)+MAX_PATH] ;
	sprintf(fullpath, "%suserhash.dat", appdir) ;

	FILE* hashfile = fopen(fullpath, "rb") ;

	delete[] fullpath ;
	if (hashfile) { 
		if (!fread(userhash, sizeof(userhash), 1, hashfile)) {
            CreateUserHash();
		}
		fclose(hashfile) ;
	}
	// End of Saving HashID 
	memcpy(&prefs->userhash,&userhash,16);

	WINDOWPLACEMENT defaultWPM;
	defaultWPM.length = sizeof(WINDOWPLACEMENT);
	defaultWPM.rcNormalPosition.left=10;defaultWPM.rcNormalPosition.top=10;
	defaultWPM.rcNormalPosition.right=700;defaultWPM.rcNormalPosition.bottom=500;
	defaultWPM.showCmd=0;
	prefs->EmuleWindowPlacement=defaultWPM;

	Save();
}

uint16 CPreferences::GetMaxDownload(){
	//dont be a Lam3r :)
	uint16 nMaxUp = GetMaxUpload();
	uint16 nMaxDown = GetMaxDownloadValue();
	if( nMaxUp < 4 )
		return ( (nMaxUp * 3 < nMaxDown) ? nMaxUp * 3 : nMaxDown);
	return ( ((nMaxUp < 10) && (nMaxUp * 4 < nMaxDown)) ? nMaxUp * 4 : nMaxDown);
}

bool CPreferences::Save(){

	bool error = false;
	char* fullpath = new char[strlen(appdir)+MAX_PATH]; // i_a
	sprintf(fullpath,"%spreferences.dat",appdir);

	FILE* preffile = fopen(fullpath,"wb");
	prefsExt->version = PREFFILE_VERSION;

	delete[] fullpath;
	if (preffile){
		prefsExt->version=PREFFILE_VERSION;
		prefsExt->EmuleWindowPlacement=prefs->EmuleWindowPlacement;
		memcpy(&prefsExt->userhash,&prefs->userhash,16);

		error = fwrite(prefsExt,sizeof(Preferences_Ext_Struct),1,preffile);
		fclose(preffile);
	}
	else
		error = true;

	// Saving HashID 
	fullpath = new char[strlen(appdir)+MAX_PATH] ;
	sprintf(fullpath, "%suserhash.dat", appdir) ;

	FILE* hashfile = fopen(fullpath, "wb") ;

	delete[] fullpath ;
	if (hashfile) {
		error = fwrite(prefs->userhash, sizeof(prefs->userhash), 1, hashfile) ;
		fclose(hashfile) ;
	}
	else
		error = true ;
	// End of Saving HashID

	SavePreferences();

	fullpath = new char[strlen(appdir)+MAX_PATH]; // i_a
	sprintf(fullpath,"%sshareddir.dat",appdir);
	CStdioFile sdirfile;
	if (sdirfile.Open(fullpath,CFile::modeCreate|CFile::modeWrite)){

		for (POSITION pos = shareddir_list.GetHeadPosition();pos != 0;shareddir_list.GetNext(pos)){
			sdirfile.WriteString(shareddir_list.GetAt(pos).GetBuffer());
			sdirfile.Write("\n",1);
		}
		sdirfile.Close();
	}
	else
		error = true;

	// start added by InterCeptor (multiple tempdirs) 19.11.02
	//fullpath = new char[strlen(appdir)+MAX_PATH]; [FoRcHa] (we already have one)
	sprintf(fullpath,"%stempdir.dat",appdir);
	CStdioFile tdirfile;
	if (tdirfile.Open(fullpath,CFile::modeCreate|CFile::modeWrite)){
		for (POSITION pos = tempdir_list.GetHeadPosition();pos != 0;tempdir_list.GetNext(pos)){
			tdirfile.WriteString(tempdir_list.GetAt(pos).GetBuffer());
			tdirfile.Write("\n",1);
		}
		tdirfile.Close();
	}
	else
		error = true;
	// end added by InterCeptor (multiple tempdirs) 19.11.02
	delete[] fullpath;
	CreateDirectory(GetIncomingDir(),0);
	CreateDirectory(GetTempDir(),0);
	//CreateDirectory(GetVlcDir(),0);		//Cax2 - not needed anymore with official video player
	return error;
}

void CPreferences::CreateUserHash(){
	for (int i = 0;i != 8; i++){ 
		uint16	random = rand();
		memcpy(&userhash[i*2],&random,2);
	}
	// mark as emule client. that will be need in later version
	userhash[5] = 14;
	userhash[14] = 111;
}

int CPreferences::GetColumnWidth(Table t, int index) const {
	switch(t) {
	case tableDownload:
		return prefs->downloadColumnWidths[index];
	case tableUpload:
		return prefs->uploadColumnWidths[index];
	case tableQueue:
		return prefs->queueColumnWidths[index];
	case tableSearch:
		return prefs->searchColumnWidths[index];
	case tableShared:
		return prefs->sharedColumnWidths[index];
	case tableServer:
		return prefs->serverColumnWidths[index];
	case tableIrc:
		return prefs->ircColumnWidths[index];
	}
	return 0;
}

void CPreferences::SetColumnWidth(Table t, int index, int width) {
	switch(t) {
	case tableDownload:
		prefs->downloadColumnWidths[index] = width;
		break;
	case tableUpload:
		prefs->uploadColumnWidths[index] = width;
		break;
	case tableQueue:
		prefs->queueColumnWidths[index] = width;
		break;
	case tableSearch:
		prefs->searchColumnWidths[index] = width;
		break;
	case tableShared:
		prefs->sharedColumnWidths[index] = width;
		break;
	case tableServer:
		prefs->serverColumnWidths[index] = width;
		break;
	case tableIrc:
		prefs->ircColumnWidths[index] = width;
		break;
	}
}

BOOL CPreferences::GetColumnHidden(Table t, int index) const {
	switch(t) {
	case tableDownload:
		return prefs->downloadColumnHidden[index];
	case tableUpload:
		return prefs->uploadColumnHidden[index];
	case tableQueue:
		return prefs->queueColumnHidden[index];
	case tableSearch:
		return prefs->searchColumnHidden[index];
	case tableShared:
		return prefs->sharedColumnHidden[index];
	case tableServer:
		return prefs->serverColumnHidden[index];
	case tableIrc:
		return prefs->ircColumnHidden[index];
	}
	return FALSE;
}

void CPreferences::SetColumnHidden(Table t, int index, BOOL bHidden) {
	switch(t) {
	case tableDownload:
		prefs->downloadColumnHidden[index] = bHidden;
		break;
	case tableUpload:
		prefs->uploadColumnHidden[index] = bHidden;
		break;
	case tableQueue:
		prefs->queueColumnHidden[index] = bHidden;
		break;
	case tableSearch:
		prefs->searchColumnHidden[index] = bHidden;
		break;
	case tableShared:
		prefs->sharedColumnHidden[index] = bHidden;
		break;
	case tableServer:
		prefs->serverColumnHidden[index] = bHidden;
		break;
	case tableIrc:
		prefs->ircColumnHidden[index] = bHidden;
		break;
	}
}

int CPreferences::GetColumnOrder(Table t, int index) const {
	switch(t) {
	case tableDownload:
		return prefs->downloadColumnOrder[index];
	case tableUpload:
		return prefs->uploadColumnOrder[index];
	case tableQueue:
		return prefs->queueColumnOrder[index];
	case tableSearch:
		return prefs->searchColumnOrder[index];
	case tableShared:
		return prefs->sharedColumnOrder[index];
	case tableServer:
		return prefs->serverColumnOrder[index];
	case tableIrc:
		return prefs->ircColumnOrder[index];
	}
	return 0;
}

void CPreferences::SetColumnOrder(Table t, INT *piOrder) {
	switch(t) {
	case tableDownload:
		memcpy(prefs->downloadColumnOrder, piOrder, sizeof(prefs->downloadColumnOrder));
		break;
	case tableUpload:
		memcpy(prefs->uploadColumnOrder, piOrder, sizeof(prefs->uploadColumnOrder));
		break;
	case tableQueue:
		memcpy(prefs->queueColumnOrder, piOrder, sizeof(prefs->queueColumnOrder));
		break;
	case tableSearch:
		memcpy(prefs->searchColumnOrder, piOrder, sizeof(prefs->searchColumnOrder));
		break;
	case tableShared:
		memcpy(prefs->sharedColumnOrder, piOrder, sizeof(prefs->sharedColumnOrder));
		break;
	case tableServer:
		memcpy(prefs->serverColumnOrder, piOrder, sizeof(prefs->serverColumnOrder));
		break;
	case tableIrc:
		memcpy(prefs->ircColumnOrder, piOrder, sizeof(prefs->ircColumnOrder));
		break;
	}
}

CPreferences::~CPreferences(){
	delete[] appdir;
	delete prefs;
	delete prefsExt;
}

int CPreferences::GetRecommendedMaxConnections() {
	int iRealMax = ::GetMaxConnections();
	if(iRealMax == -1 || iRealMax > 520)
		return 500;

	if(iRealMax < 20)
		return iRealMax;

	if(iRealMax <= 256)
		return iRealMax - 10;

	return iRealMax - 20;
}

void CPreferences::SavePreferences(){
	CString buffer;
	char* fullpath = new char[strlen(appdir)+MAX_PATH]; // i_a
	sprintf(fullpath,"%spreferences.ini",appdir);
	
	CIni ini( fullpath, "eMule" );

	delete[] fullpath;

	//useful!
	ini.WriteString("AppVersion", CURRENT_VERSION_LONG);

	buffer.Format("%s",m_sNick.Get().c_str());
	ini.WriteString("Nick",buffer);

	buffer.Format("%s", m_sIncomingDir.Get().c_str());
	ini.WriteString("IncomingDir",buffer );

	buffer.Format("%s", m_sTempDir.Get().c_str());
	ini.WriteString("TempDir",buffer );

	//buffer.Format("%s", m_sVlcDir.Get().c_str());	//Cax2 we don't save this anymore. m_sVlcDir to be removed in 1b & above
	//ini.WriteString("VLCDir",buffer);
	buffer.Format("%s", m_sVideoPlayer.Get().c_str());	//Cax2 - official mule code.
	ini.WriteString("VideoPlayer",buffer );

	ini.WriteInt("MaxUpload", m_nMaxUpload.Get());
	ini.WriteInt("MaxDownload", m_nMaxDownload.Get());
	ini.WriteInt("MaxConnections", m_nMaxConnections.Get());
	ini.WriteInt("DeadServerAction", m_nDeadServer.Get());
	ini.WriteInt("Port", m_nPort.Get());
	ini.WriteInt("UDPPort", m_nUDPPort.Get());
	ini.WriteInt("MaxSourcesPerFile", m_nMaxSourcePerFile.Get());
	//ini.WriteInt("MaxSourcesPerFileSoft",prefs->maxsourceperfilesoft);
	//ini.WriteInt("MaxSourcesPerFileUDP",prefs->maxsourceperfileudp );
	ini.WriteWORD("Language", m_nLanguageID.Get());
	ini.WriteInt("SeeShare", m_nSeeShares.Get());
	//--- xrmb:partTrafficPrefs ---
	ini.WriteBool("PartTrafficEnabled",prefs->m_pt_useit);
	ini.WriteInt("UploadbarStyle",prefs->m_upbar_st);
	ini.WriteInt("UploadbarColor",prefs->m_upbar_cl);
	//--- :xrmb ---
	ini.WriteInt("ToolTipDelay",prefs->m_iToolDelayTime);
	ini.WriteInt("QueueSize", prefs->queuesize) ; // SeeWen QUEUESIZE
	//ini.WriteInt("RemoteQueueLimit", prefs->remotequeuelimit); //remote QR 
	ini.WriteBool("UseRemoteQueueLimit", m_bUseRQlimit.Get());
	ini.WriteInt("StatGraphsInterval", m_nTrafficOMeterInterval.Get());
	ini.WriteInt("StatsInterval", m_nStatsInterval.Get());
	ini.WriteInt("StatsColumnWidth", m_nStatsColumnWidth.Get());
	ini.WriteInt("DetailColumnWidth", m_nDetailColumnWidth.Get());
	ini.WriteString("StatsListExpanded", m_sStatsListExpanded.Get().c_str());
	ini.WriteInt("DownloadCapacity", m_nMaxGraphDownloadRate.Get());
	ini.WriteInt("UploadCapacity", m_nMaxGraphUploadRate.Get());
	ini.WriteInt("DeadServerRetries",prefs->deadserverretries);
	ini.WriteInt("SplitterbarPosition",prefs->splitterbarPosition);
	ini.WriteInt("VariousStatisticsMaxValue",prefs->statsMax);
	ini.WriteInt("StatsAverageMinutes",prefs->statsAverageMinutes);
	ini.WriteInt("MaxConnectionsPerFiveSeconds",prefs->MaxConperFive);
	ini.WriteInt("MaximumUploadClients",prefs->maxupclientsallowed);
	ini.WriteInt("MinimumUploadClients",prefs->minupclientsallowed);
	ini.WriteInt("UploadClientDatarate",prefs->uploadclientdatarate);
	ini.WriteInt("BadClientBanTime",prefs->badclientbantime);
	ini.WriteInt("BadClientMinRequestTime",prefs->minrequesttime);
	ini.WriteInt("BadClientMinRequestNum",prefs->badclientban);
	ini.WriteInt("SLS_RestoreSourcesTime",prefs->slsrestoresourcestime); // reCDVst
	ini.WriteInt("SLS_LoadQuantities",prefs->slsloadquantities); // DonGato
	ini.WriteInt("SLS_WhenSourcesOutdated",prefs->slsoutdated); // reCDVst
	ini.WriteInt("SLS_MaxSourcesPerFile",prefs->slsmaxsourcesperfile); // reCDVst
	ini.WriteInt("SLS_Enable",prefs->slsenable); // DonGato

	ini.WriteBool("Reconnect", m_bReconnect.Get());
	ini.WriteBool("Scoresystem", m_nScoreSystem.Get());

	// Added by Tarod
	ini.WriteBool("UAP", prefs->UAP);
	ini.WriteBool("DAP", prefs->DAP);
	ini.WriteBool("DisableXS", prefs->disablexs);
	ini.WriteBool("StartDownloadPaused", prefs->startdownloadpaused);
	ini.WriteBool("DownloadPausedOnComplete", prefs->downloadpausedonclomplete);
	ini.WriteBool("RemoveNNS", prefs->remove_nns);
	
	ini.WriteString("ToolbarSettings", prefs->toolbarsettings); // Added by reCDVst
	buffer.Format("%s", m_sToolbarBitmap.Get().c_str());
	ini.WriteString("ToolbarBitmap",buffer);
	buffer.Format("%s", m_sToolbarBitmapFolder.Get().c_str());
	ini.WriteString("ToolbarBitmapFolder",buffer);
	ini.WriteInt("ToolbarLabels", m_nToolbarLabels.Get());
	ini.WriteBool("Serverlist", m_bAutoServerList.Get());
	ini.WriteBool("UpdateNotify", m_bUpdateNotify.Get());
	ini.WriteBool("MinimizeToTray", m_bMinToTray.Get());
	ini.WriteBool("AddServersFromConnections", m_bAddServersFromConn.Get());
	ini.WriteBool("Splashscreen", m_bSplashScreen.Get());
	ini.WriteBool("BringToFront",prefs->bringtoforeground);
	ini.WriteBool("TransferDoubleClick", m_nTransferDoubleclick.Get());
	ini.WriteBool("BeepOnError", m_bBeepOnError.Get());
	ini.WriteBool("ConfirmExit", m_bConfirmExit.Get());
	ini.WriteBool("FilterBadIPs", m_bFilterBadIP.Get());
	ini.WriteBool("Autoconnect", m_bAutoConnect.Get());
	ini.WriteBool("OnlineSignature", m_bOnlineSig.Get());
	ini.WriteBool("StartupMinimized",prefs->startMinimized);
	ini.WriteBool("SafeConnect",prefs->SafeConnect);
	ini.WriteBool("ToolbarSpeedMeter",prefs->showToolbarSpeedMeter);
	ini.WriteBool("CloseToTray",prefs->closeToTray);
	ini.WriteBool("DetailsOnClick",prefs->DetailsOnClick);
	ini.WriteInt("SrvConTimeout", prefs->SrvConTimeout);
	ini.WriteBool("SlowComplete",prefs->SlowComplete);
	ini.WriteInt("SlowCompleteBlockSize", prefs->SlowCompleteBlockSize);
	ini.WriteBool("ShowRatesOnTitle",prefs->showRatesInTitle);

	// Barry - New properties...
    ini.WriteBool("AutoConnectStaticOnly", m_bAutoConnectStaticOnly.Get());  
    ini.WriteBool("GlobalSearch", prefs->globalsearch);  
    ini.WriteInt ("3DDepth", m_nDepth3D.Get()); 

	// Added by enkeyDEV
	ini.WriteBool("NotifyOnDownload",prefs->useDownloadNotifier);
	ini.WriteBool("NotifyOnDownloadAdd",prefs->useDownloadAddNotifier);
	ini.WriteBool("NotifyOnChat",prefs->useChatNotifier);		  
	ini.WriteBool("NotifyOnLog",prefs->useLogNotifier);
	ini.WriteBool("NotifierUseSound",prefs->useSoundInNotifier);
	ini.WriteBool("NotifierPopEveryChatMessage",prefs->notifierPopsEveryChatMsg);
	ini.WriteBool("NotifyOnImportantError", prefs->notifierImportantError);
	buffer.Format("%s",prefs->notifierSoundFilePath);
	ini.WriteString("NotifierSoundPath",buffer);

	ini.WriteString("MessageFilter",prefs->messageFilter);
	ini.WriteString("TxtEditor",prefs->TxtEditor);
	ini.WriteString("IRCServer",prefs->m_sircserver);
	ini.WriteString("IRCNick",prefs->m_sircnick);
	ini.WriteBool("IRCAddTimestamp", prefs->m_bircaddtimestamp);
	ini.WriteString("IRCFilterName", prefs->m_sircchannamefilter);
	ini.WriteInt("IRCFilterUser", prefs->m_iircchanneluserfilter);
	ini.WriteBool("IRCUseFilter", prefs->m_bircusechanfilter);
	ini.WriteString("IRCPerformString", prefs->m_sircperformstring);
	ini.WriteBool("IRCUsePerform", prefs->m_bircuseperform);
	ini.WriteBool("IRCListOnConnect", prefs->m_birclistonconnect);
	ini.WriteBool("IRCAcceptLinks", prefs->m_bircacceptlinks);
	ini.WriteBool("IRCIgnoreInfoMessage", prefs->m_bircignoreinfomessage);
	ini.WriteBool("IRCIgnoreEmuleProtoInfoMessage", prefs->m_bircignoreemuleprotoinfomessage);
	ini.WriteBool("SmartIdCheck", prefs->smartidcheck);

	ini.WriteBool("UseSortingDefaults",prefs->useSort);  	//sortprefs Cax2
	ini.WriteBool("Use2ndSortSources",prefs->useSrcSortCol2);
	ini.WriteInt("SortServersCode",prefs->serverSortCol);	
	ini.WriteInt("SortUploadsCode",prefs->uploadSortCol);
	ini.WriteInt("SortQueueCode",prefs->queueSortCol);
	ini.WriteInt("SortSearchCode",prefs->searchSortCol);
	ini.WriteInt("SortIrcCode",prefs->ircSortCol);
	ini.WriteInt("SortFilesCode",prefs->fileSortCol);
	ini.WriteInt("SortDownloadsCode",prefs->downloadSortCol);
	ini.WriteInt("1stSortSourcesCode",prefs->srcSortCol1);
	ini.WriteInt("2ndSortSourcesCode",prefs->srcSortCol2);

	// DonGato
	ini.WriteBool("CommunityEnabled", prefs->CommunityEnabled);
	ini.WriteString("CommunityString", prefs->CommunityString);
	ini.WriteBool("CommunityNoBan", prefs->CommunityNoBan);
	ini.WriteInt("NotificationDisplayTime",prefs->NotificationDisplayTime);
	ini.WriteInt("NotificationFontSize",prefs->NotificationFontSize);
	ini.WriteInt("Profile", prefs->Profile);
	ini.WriteInt("PriorityHigh", prefs->PriorityHigh);
	ini.WriteInt("PriorityLow", prefs->PriorityLow);
	ini.WriteBool("RestartWaiting", prefs->RestartWaiting);
	ini.WriteBool("LogToFile", prefs->LogToFile);
	ini.WriteBool("TransferFullChunks", prefs->TransferFullChunks);
	ini.WriteBool("BugReport", prefs->BugReport);
	ini.WriteBool("DisableXSUpTo", prefs->DisableXSUpTo);
	ini.WriteInt("XSUpTo", prefs->XSUpTo);
	ini.WriteString("UsedFont", prefs->UsedFont);
	ini.WriteInt("FontSize",prefs->FontSize);
	ini.WriteInt("HashingPriority",prefs->HashingPriority);
	ini.WriteBool("UploadParts", prefs->UploadParts);
	ini.WriteBool("A4AFStringEnabled", m_bA4AFStringEnabled.Get());
	ini.WriteBool("BanMessageEnabled", m_bBanMessageEnabled.Get());
	ini.WriteInt("SearchMethod", m_nSearchMethod.Get());
	ini.WriteBool("AutoTakeED2KLinks", m_bAutoTakeLinks.Get()); 
	ini.WriteInt("FileBufferSizePref", m_nFileBufferSize.Get());
	ini.WriteInt("MainProcessPriority", m_nMainProcessPriority.Get());

	ini.WriteBool("NSEnabled", m_bNSEnabled.Get());
	ini.WriteBool("NSExceptSat", m_bNSExceptSat.Get());
	ini.WriteBool("NSExceptSun", m_bNSExceptSun.Get());
	ini.WriteInt("NSStart",m_nNSStart.Get());
	ini.WriteInt("NSEnd",m_nNSEnd.Get());
	ini.WriteInt("NSDayUp",m_nNSDayUpload.Get());
	ini.WriteInt("NSDayDown",m_nNSDayDownload.Get());
	ini.WriteInt("NSNightUp",m_nNSNightUpload.Get());
	ini.WriteInt("NSNightDown",m_nNSNightDownload.Get());

	// EpicStruggle
	ini.WriteBool("MultipleInstances", prefs->m_multiple);

	ini.WriteBool("Verbose", prefs->m_bVerbose);
	ini.WriteBool("PreviewPrio", prefs->m_bpreviewprio);
	ini.WriteBool("UpdateQueueList", prefs->m_bupdatequeuelist);
	ini.WriteBool("ManualHighPrio", prefs->m_bmanualhighprio);
	//ini.WriteBool("StartNextFile", prefs->m_bstartnextfile); //duplicates downloadpausedoncomplete
	ini.WriteBool("ShowOverhead", m_bShowOverhead.Get());	//Cax2 re-enabled this preference
	ini.WriteBool("VideoPreviewBackupped", m_bBackupPreview.Get());	//Cax2 string kept for compatibility with off client

	ini.SerGet(false, prefs->downloadColumnWidths,
		ELEMENT_COUNT(prefs->downloadColumnWidths), "DownloadColumnWidths");
	ini.SerGet(false, prefs->downloadColumnHidden,
		ELEMENT_COUNT(prefs->downloadColumnHidden), "DownloadColumnHidden");
	ini.SerGet(false, prefs->downloadColumnOrder,
		ELEMENT_COUNT(prefs->downloadColumnOrder), "DownloadColumnOrder");
	ini.SerGet(false, prefs->uploadColumnWidths,
		ELEMENT_COUNT(prefs->uploadColumnWidths), "UploadColumnWidths");
	ini.SerGet(false, prefs->uploadColumnHidden,
		ELEMENT_COUNT(prefs->uploadColumnHidden), "UploadColumnHidden");
	ini.SerGet(false, prefs->uploadColumnOrder,
		ELEMENT_COUNT(prefs->uploadColumnOrder), "UploadColumnOrder");
	ini.SerGet(false, prefs->queueColumnWidths,
		ELEMENT_COUNT(prefs->queueColumnWidths), "QueueColumnWidths");
	ini.SerGet(false, prefs->queueColumnHidden,
		ELEMENT_COUNT(prefs->queueColumnHidden), "QueueColumnHidden");
	ini.SerGet(false, prefs->queueColumnOrder,
		ELEMENT_COUNT(prefs->queueColumnOrder), "QueueColumnOrder");
	ini.SerGet(false, prefs->searchColumnWidths,
		ELEMENT_COUNT(prefs->searchColumnWidths), "SearchColumnWidths");
	ini.SerGet(false, prefs->searchColumnHidden,
		ELEMENT_COUNT(prefs->searchColumnHidden), "SearchColumnHidden");
	ini.SerGet(false, prefs->searchColumnOrder,
		ELEMENT_COUNT(prefs->searchColumnOrder), "SearchColumnOrder");
	ini.SerGet(false, prefs->sharedColumnWidths,
		ELEMENT_COUNT(prefs->sharedColumnWidths), "SharedColumnWidths");
	ini.SerGet(false, prefs->sharedColumnHidden,
		ELEMENT_COUNT(prefs->sharedColumnHidden), "SharedColumnHidden");
	ini.SerGet(false, prefs->sharedColumnOrder,
		ELEMENT_COUNT(prefs->sharedColumnOrder), "SharedColumnOrder");
	ini.SerGet(false, prefs->serverColumnWidths,
		ELEMENT_COUNT(prefs->serverColumnWidths), "ServerColumnWidths");
	ini.SerGet(false, prefs->serverColumnHidden,
		ELEMENT_COUNT(prefs->serverColumnHidden), "ServerColumnHidden");
	ini.SerGet(false, prefs->serverColumnOrder,
		ELEMENT_COUNT(prefs->serverColumnOrder), "ServerColumnOrder");
	ini.SerGet(false, prefs->ircColumnWidths,
		ELEMENT_COUNT(prefs->ircColumnWidths), "IrcColumnWidths");
	ini.SerGet(false, prefs->ircColumnHidden,
		ELEMENT_COUNT(prefs->ircColumnHidden), "IrcColumnHidden");
	ini.SerGet(false, prefs->ircColumnOrder,
		ELEMENT_COUNT(prefs->ircColumnOrder), "IrcColumnOrder");

	// Barry - Provide a mechanism for all tables to store/retrieve sort order
	ini.WriteInt("TableSortItemDownload", prefs->tableSortItemDownload);
	ini.WriteInt("TableSortItemUpload", prefs->tableSortItemUpload);
	ini.WriteInt("TableSortItemQueue", prefs->tableSortItemQueue);
	ini.WriteInt("TableSortItemSearch", prefs->tableSortItemSearch);
	ini.WriteInt("TableSortItemShared", prefs->tableSortItemShared);
	ini.WriteInt("TableSortItemServer", prefs->tableSortItemServer);
	ini.WriteBool("TableSortAscendingDownload", prefs->tableSortAscendingDownload);
	ini.WriteBool("TableSortAscendingUpload", prefs->tableSortAscendingUpload);
	ini.WriteBool("TableSortAscendingQueue", prefs->tableSortAscendingQueue);
	ini.WriteBool("TableSortAscendingSearch", prefs->tableSortAscendingSearch);
	ini.WriteBool("TableSortAscendingShared", prefs->tableSortAscendingShared);
	ini.WriteBool("TableSortAscendingServer", prefs->tableSortAscendingServer);

	CString buffer2;
	for (int i=0;i<15;i++) {
		buffer.Format("%I64Lu",GetStatsColor(i));
		buffer2.Format("StatColor%i",i);
		ini.WriteString(buffer2,buffer);
	}
	ini.WriteInt("ConnectionsGraphRatio", m_nGraphRatio.Get()); // propstats Cax2

	// Added By Maverick
	ini.WriteBool("MlDonkeyCM",prefs->MlDonkeyCM);

	// Added by Tarod -saving HASH of A4AF auto file
	char hex_hash[33];
	hex_hash[0] = '\0';
	for (uint16 i = 0; i < 16; i++) // hmm I wonder if there is a standart function for this
		sprintf(hex_hash, "%s%02X", hex_hash, prefs->A4AF_file_hash[i]);
	ini.WriteString("A4AFAutoFileHash", CString(hex_hash));	

	// Maverick
	ini.WriteBool("EnablePassword",prefs->proxy.EnablePassword,"Proxy");
	ini.WriteBool("EnableProxy",prefs->proxy.UseProxy,"Proxy");
	ini.WriteString("Name",prefs->proxy.name,"Proxy");
	ini.WriteString("Password",prefs->proxy.password,"Proxy");
	ini.WriteString("User",prefs->proxy.user,"Proxy");
	ini.WriteInt("Port",prefs->proxy.port,"Proxy");
	ini.WriteInt("Type",prefs->proxy.type,"Proxy");

	buffer.Format("%I64Lu", m_nTotalDownloadedBytes.Get());
	ini.WriteString("TotalDownloadedBytes",buffer ,"Statistics");

	buffer.Format("%I64Lu", m_nTotalUploadedBytes.Get());
	ini.WriteString("TotalUploadedBytes",buffer ,"Statistics");

	// Web Server
	ini.WriteString("Password", CString(m_sWebPassword.Get().c_str()), "WebServer");	
	ini.WriteInt("Port", m_nWebPort.Get(), "WebServer");
	ini.WriteBool("Enabled", m_bWebEnabled.Get(), "WebServer");
	ini.WriteBool("UseGzip", m_bWebUseGzip.Get(), "WebServer");
	ini.WriteInt("PageRefreshTime", m_nWebPageRefresh.Get(), "WebServer");

	//////////////////////////////////////////////////////////////////
	// LANCAST (moosetea) - Save the settings
	
	char t_strLanCastIP[17];
	sprintf(t_strLanCastIP,"%i.%i.%i.%i",(uint8)m_LancastIP.Get(),(uint8)(m_LancastIP.Get()>>8),(uint8)(m_LancastIP.Get()>>16),(uint8)(m_LancastIP.Get()>>24));

	char t_strLanCastSubnet[17];
	sprintf(t_strLanCastSubnet,"%i.%i.%i.%i",(uint8)m_LancastSubnet.Get(),(uint8)(m_LancastSubnet.Get()>>8),(uint8)(m_LancastSubnet.Get()>>16),(uint8)(m_LancastSubnet.Get()>>24));
				
	ini.WriteBool("Enabled",m_LancastEnabled.Get(),"Lancast");
	ini.WriteBool("UseDefaultIP",m_UseDefaultIP.Get(),"Lancast");
	ini.WriteString("IPAddress",t_strLanCastIP,"Lancast");
	ini.WriteString("Subnet",t_strLanCastSubnet,"Lancast");
	ini.WriteInt("Port",m_LancastPort.Get(),"Lancast");

	//////////////////////////////////////////////////////////////////

	//START enkeyDEV(kei-kun) -Messaging preferences-
	ini.WriteInt("AcceptMessageFrom",m_nAcceptMessagesFrom.Get());	
	ini.WriteString("AwayStateMessage", CString(m_sAwayStateMessage.Get().c_str()));
	//END enkeyDEV(kei-kun) -Messaging preferences-
}

DWORD CPreferences::DefaultStatsColor(int index){
	switch(index)
	{
		case 0 : return RGB(0,0,64);
		case 1 : return RGB(192,192,255);
		case 2 : return RGB(128, 255, 128);
		case 3 : return RGB(0, 210, 0);
		case 4 : return RGB(0, 128, 0);
		case 5 : return RGB(255, 128, 128);
		case 6 : return RGB(200, 0, 0);
		case 7 : return RGB(140, 0, 0);
		case 8 : return RGB(150, 150, 255);
		case 9 : return RGB(192,   0, 192);
		case 10 : return RGB(255, 192, 255);
		case 11 : return RGB(255, 255, 128);
		default : return RGB(255, 255, 255);
	}
}

void CPreferences::LoadPreferences(){
//	CSingleLock PrefLock(&m_Pref_Mutex,TRUE);
	char buffer[200];

	//--- Quick hack to add version tag to preferences.ini-file and solve the issue with the FlatStatusBar tag...
	CString strFileName;
	strFileName.Format("%spreferences.ini", appdir);
	CIni* pIni = new CIni(strFileName, "eMule");
	CString strCurrVersion, strPrefsVersion;
	strCurrVersion = CURRENT_VERSION_LONG;
	strPrefsVersion = pIni->GetString("AppVersion");
	delete pIni;
	CFileFind findFileName;

	if ((strCurrVersion != strPrefsVersion) && findFileName.FindFile(strFileName))
	{
		CFile file;
		CFileFind findNewName;
		CString strNewName;
		//Cax2 we can add the code to find our version's setting, if the ini file is from a different version...
		strNewName.Format("%spreferences[%s].ini", appdir,strPrefsVersion);	
		if (findNewName.FindFile(strNewName))
			file.Remove(strNewName);
		file.Rename(strFileName, strNewName);
		strFileName = strNewName;
	}

	CIni ini(strFileName, "eMule");
	//--- end Ozon :)

	CString strDefNick;	// random nick [FoRcHa]
	srand((unsigned)time(NULL));
	for(int i = 0; i < 6; i++)
		buffer[i] = rand() % ('z' - 'a') + 'a';
	buffer[i] = '\0';
	strDefNick.Format("%s [eMule Plus v%s]", buffer, CURRENT_VERSION_LONG);
	m_sNick.Put(ini.GetString("Nick",strDefNick).GetBuffer(0));
	
	sprintf(buffer,"%sIncoming",appdir);
	m_sIncomingDir.Put(ini.GetString("IncomingDir",buffer).GetBuffer(0));

	sprintf(buffer,"%sTemp",appdir);
	m_sTempDir.Put(ini.GetString("TempDir",buffer).GetBuffer(0));

	sprintf(buffer,"%sVLC",appdir);
	m_sVlcDir.Put(ini.GetString("VLCDir",buffer).GetBuffer(0));
	sprintf(buffer,"%s\\vlc.exe",GetVlcDir());		//Cax2 - for backward compatibility
	m_sVideoPlayer.Put(ini.GetString("VideoPlayer",buffer).GetBuffer(0));		//Cax2 - replace buffer with "" in later (non b-compatible) versions
	m_nMaxGraphUploadRate.Put(ini.GetInt("UploadCapacity",16));
	m_nMaxGraphDownloadRate.Put(ini.GetInt("DownloadCapacity",96));
	m_nMaxUpload.Put(ini.GetInt("MaxUpload",UNLIMITED));		//Cax2 new limits enforced on old settings
	if (m_nMaxUpload.Get()!=UNLIMITED && m_nMaxUpload.Get()>m_nMaxGraphUploadRate.Get())
		m_nMaxUpload.Put(m_nMaxGraphDownloadRate.Get());
	m_nMaxDownload.Put(ini.GetInt("MaxDownload",UNLIMITED));
	if (m_nMaxDownload.Get()!=UNLIMITED && m_nMaxDownload.Get()>m_nMaxGraphDownloadRate.Get())
		m_nMaxDownload.Put(m_nMaxGraphDownloadRate.Get());
	m_nMaxConnections.Put(ini.GetInt("MaxConnections",GetRecommendedMaxConnections()));
	m_nDeadServer.Put(ini.GetInt("DeadServerAction",2));
	m_nPort.Put(ini.GetInt("Port",4662));
	m_nUDPPort.Put(ini.GetInt("UDPPort",m_nPort.Get() + 10));
	m_nMaxSourcePerFile.Put(ini.GetInt("MaxSourcesPerFile",400));
	//prefs->maxsourceperfilesoft=ini.GetInt("MaxSourcesPerFileSoft",360);
	//prefs->maxsourceperfileudp=ini.GetInt("MaxSourcesPerFileUDP",100);
	prefs->queuesize=ini.GetInt("QueueSize", MIN_QUEUE*2); // SeeWen QUEUESIZE
	if(prefs->queuesize > MAX_QUEUE)
		prefs->queuesize = MAX_QUEUE;
	else if(prefs->queuesize < MIN_QUEUE)
		prefs->queuesize = MIN_QUEUE;
	m_nRQlimit.Put((prefs->queuesize >= 2000)?prefs->queuesize:2000); //Cax2 - remote queue limit linked to queue size, but no lower than 2000
	m_bUseRQlimit.Put(ini.GetBool("UseRemoteQueueLimit"));
	m_nLanguageID.Put(ini.GetWORD("Language",0));
	m_nSeeShares.Put((enSeeShare)ini.GetInt("SeeShare",SEE_SHARE_EVERYBODY));
	//--- xrmb:partTrafficPrefs ---
	prefs->m_pt_useit=ini.GetBool("PartTrafficEnabled",false);	//Cax2
	prefs->m_upbar_st=ini.GetInt("UploadbarStyle", 2);
	prefs->m_upbar_cl=ini.GetInt("UploadbarColor", 0);
	//--- :xrmb ---
	prefs->m_iToolDelayTime=ini.GetInt("ToolTipDelay",3);
	m_nTrafficOMeterInterval.Put(ini.GetInt("StatGraphsInterval",3));
	m_nStatsInterval.Put(ini.GetInt("statsInterval",5));
	m_nStatsColumnWidth.Put(ini.GetInt("StatsColumnWidth", 150));
	m_nDetailColumnWidth.Put(ini.GetInt("DetailColumnWidth", 150));
	
	m_sStatsListExpanded.Put(ini.GetString("StatsListExpanded", "").GetBuffer(0)); // missing entrys get expanded ("" = all expanded)

	prefs->deadserverretries=ini.GetInt("DeadServerRetries",1);
	prefs->splitterbarPosition=ini.GetInt("SplitterbarPosition",75);
	prefs->statsMax=ini.GetInt("VariousStatisticsMaxValue",100);
	prefs->statsAverageMinutes=ini.GetInt("StatsAverageMinutes",5);
	prefs->MaxConperFive=ini.GetInt("MaxConnectionsPerFiveSeconds",GetDefaultMaxConperFive());
	prefs->showRatesInTitle=ini.GetBool("ShowRatesOnTitle",false);

	// DonGato
	int nMax = (LOBYTE(GetWindowsVersion()) == 0x04)?100:200;
	if(prefs->MaxConperFive > nMax) {
		prefs->MaxConperFive = nMax;
	} else {
		if(prefs->MaxConperFive < 5)
			prefs->MaxConperFive = 5;
	}

	prefs->badclientbantime=ini.GetInt("BadClientBanTime",3600000);
	if(prefs->badclientbantime > 18000000)
		prefs->badclientbantime = 18000000;
	
	prefs->minrequesttime=ini.GetInt("BadClientMinRequestTime",590000);
	if(prefs->minrequesttime > 590000)
		prefs->minrequesttime = 590000;
	
	prefs->badclientban=ini.GetInt("BadClientMinRequestNum",3);
	if(prefs->badclientban < 3)
		prefs->badclientban = 3;

	// Original by reCDVst updated by [DonGato]
	prefs->slsrestoresourcestime=ini.GetInt("SLS_RestoreSourcesTime",7200000);
	if(prefs->slsrestoresourcestime < 3600000)
		prefs->slsrestoresourcestime = 3600000;
	
	prefs->slsloadquantities=ini.GetInt("SLS_LoadQuantities",20);
	if(prefs->slsloadquantities > 100)
		prefs->slsloadquantities = 100;

	prefs->slsoutdated=ini.GetInt("SLS_WhenSourcesOutdated",2);
	if(prefs->slsoutdated > 10)
		prefs->slsoutdated = 10;
	
	prefs->slsenable = ini.GetBool("SLS_Enable",false);

	m_bReconnect.Put(ini.GetBool("Reconnect",true));
	m_nScoreSystem.Put(ini.GetBool("Scoresystem",true));
	
	// Added by Tarod
	prefs->UAP = ini.GetBool("UAP", true);
	prefs->DAP = ini.GetBool("DAP", true);
	prefs->disablexs = ini.GetBool("DisableXS", false);
	prefs->startdownloadpaused = ini.GetBool("StartDownloadPaused", false);
	prefs->downloadpausedonclomplete = ini.GetBool("DownloadPausedOnComplete", false);
	prefs->remove_nns = ini.GetBool("RemoveNNS", false);
	
	sprintf(prefs->toolbarsettings,"%s",ini.GetString("ToolbarSettings", "0099010203040506079908")); // Added by reCDVst
	m_sToolbarBitmap.Put(ini.GetString("ToolbarBitmap", "").GetBuffer(0)); // Added by reCDVst
	m_sToolbarBitmapFolder.Put(ini.GetString("ToolbarBitmapFolder", "").GetBuffer(0)); // Added by reCDVst
	m_nToolbarLabels.Put(ini.GetInt("ToolbarLabels",1));
	m_bAutoServerList.Put(ini.GetBool("Serverlist",false));
	m_bUpdateNotify.Put(ini.GetBool("UpdateNotify",true));
	m_bMinToTray.Put(ini.GetBool("MinimizeToTray",true));
	m_bAddServersFromConn.Put(ini.GetBool("AddServersFromConnections",true));
	m_bSplashScreen.Put(ini.GetBool("Splashscreen",true));
	prefs->bringtoforeground=ini.GetBool("BringToFront",true);
	m_nTransferDoubleclick.Put(ini.GetBool("TransferDoubleClick",true));
	m_bBeepOnError.Put(ini.GetBool("BeepOnError",true));
	m_bConfirmExit.Put(ini.GetBool("ConfirmExit",true));
	m_bFilterBadIP.Put(ini.GetBool("FilterBadIPs",true));
	m_bAutoConnect.Put(ini.GetBool("Autoconnect",false));
	m_bOnlineSig.Put(ini.GetBool("OnlineSignature",false));
	prefs->startMinimized=ini.GetBool("StartupMinimized",false);
	prefs->SafeConnect =ini.GetBool("SafeConnect",true);
	prefs->showToolbarSpeedMeter = ini.GetBool("ToolbarSpeedMeter", false);
	prefs->closeToTray=ini.GetBool("CloseToTray",false);	
	prefs->DetailsOnClick=ini.GetBool("DetailsOnClick",false);	

	prefs->SrvConTimeout=ini.GetInt("SrvConTimeout",30000);
	if(prefs->SrvConTimeout < 30000)
		prefs->SrvConTimeout = 30000;
	else if(prefs->SrvConTimeout > 360000)
		prefs->SrvConTimeout = 360000;
	
	prefs->SlowComplete=ini.GetBool("SlowComplete",false);	
	prefs->SlowCompleteBlockSize=ini.GetInt("SlowCompleteBlockSize",32);
	if(prefs->SlowCompleteBlockSize > 1024)
		prefs->SlowCompleteBlockSize = 1024;

	// Barry - New properties...
	m_bAutoConnectStaticOnly.Put(ini.GetBool("AutoConnectStaticOnly",false)); 
	prefs->globalsearch = ini.GetBool("GlobalSearch",true); 
	m_nDepth3D.Put(ini.GetInt("3DDepth", 0));

	// as a temporary converter for previous versions
	if (strPrefsVersion < "0.25a") // before 0.25a
		if (ini.GetBool("FlatStatusBar",false))
			m_nDepth3D.Put(0);
		else 
			m_nDepth3D.Put(5);

	// Added by enkeyDEV
	prefs->useDownloadAddNotifier=ini.GetBool("NotifyOnDownloadAdd",false);
	prefs->useDownloadNotifier=ini.GetBool("NotifyOnDownload",false);
	prefs->useChatNotifier=ini.GetBool("NotifyOnChat",false);
	prefs->useLogNotifier=ini.GetBool("NotifyOnLog",false);
	prefs->useSoundInNotifier=ini.GetBool("NotifierUseSound",false);
	prefs->notifierPopsEveryChatMsg=ini.GetBool("NotifierPopEveryChatMessage",false);
	prefs->notifierImportantError=ini.GetBool("NotifyOnImportantError",false);
	sprintf(prefs->notifierSoundFilePath,"%s",ini.GetString("NotifierSoundPath",""));

	sprintf(prefs->m_sircserver,"%s",ini.GetString("IRCServer","irc.liquidirc.com"));
	sprintf(prefs->m_sircnick,"%s",ini.GetString("IRCNick","eMule Plus"));
	prefs->m_bircaddtimestamp=ini.GetBool("IRCAddTimestamp",true);
	sprintf(prefs->m_sircchannamefilter,"%s",ini.GetString("IRCFilterName", "" ));
	prefs->m_bircusechanfilter=ini.GetBool("IRCUseFilter", false);
	prefs->m_iircchanneluserfilter=ini.GetInt("IRCFilterUser", 0);
	sprintf(prefs->m_sircperformstring,"%s",ini.GetString("IRCPerformString", "/join #emule" ));
	prefs->m_bircuseperform=ini.GetBool("IRCUsePerform", false);
	prefs->m_birclistonconnect=ini.GetBool("IRCListOnConnect", true);
	prefs->m_bircacceptlinks=ini.GetBool("IRCAcceptLinks", false);
	prefs->m_bircignoreinfomessage=ini.GetBool("IRCIgnoreInfoMessage", false);
	prefs->m_bircignoreemuleprotoinfomessage=ini.GetBool("IRCIgnoreEmuleProtoInfoMessage", true);
	prefs->smartidcheck=ini.GetBool("SmartIdCheck",true);
	prefs->m_bVerbose=ini.GetBool("Verbose",false);
	prefs->m_bpreviewprio=ini.GetBool("PreviewPrio",false);
	prefs->m_bupdatequeuelist=ini.GetBool("UpdateQueueList",false);
	prefs->m_bmanualhighprio=ini.GetBool("ManualHighPrio",false);
	m_bShowOverhead.Put(ini.GetBool("ShowOverhead",true));
	m_bBackupPreview.Put(ini.GetBool("VideoPreviewBackupped",false));	//Cax2 string kept for compatibility with off client

	sprintf(prefs->TxtEditor,"%s",ini.GetString("TxtEditor","notepad.exe"));
	prefs->messageFilter=ini.GetString("MessageFilter","Your client has an infinite queue");

	prefs->useSort= ini.GetBool("UseSortingDefaults",true);  	//sortprefs Cax2
	prefs->useSrcSortCol2= ini.GetBool("Use2ndSortSources",false);
	prefs->serverSortCol = ini.GetInt("SortServersCode",0);	
	prefs->uploadSortCol = ini.GetInt("SortUploadsCode",12);
	prefs->queueSortCol = ini.GetInt("SortQueueCode",105);		//Cax2 changed 16/01/03
	prefs->searchSortCol = ini.GetInt("SortSearchCode",12);
	prefs->ircSortCol = ini.GetInt("SortIrcCode",11);
	prefs->fileSortCol = ini.GetInt("SortFilesCode",0);
	prefs->downloadSortCol = ini.GetInt("SortDownloadsCode",8);
	prefs->srcSortCol1 = ini.GetInt("1stSortSourcesCode",0);
	prefs->srcSortCol2 = ini.GetInt("2ndSortSourcesCode",255);

	// DonGato
	prefs->CommunityEnabled=ini.GetBool("CommunityEnabled", false);
	sprintf(prefs->CommunityString,"%s",ini.GetString("CommunityString",""));
	prefs->CommunityNoBan=ini.GetBool("CommunityNoBan", false);
	prefs->NotificationDisplayTime=ini.GetInt("NotificationDisplayTime",4000);
	prefs->NotificationFontSize=ini.GetInt("NotificationFontSize",70);
	prefs->Profile=ini.GetInt("Profile",0);
	prefs->PriorityHigh=ini.GetInt("PriorityHigh",100);
	prefs->PriorityLow=ini.GetInt("PriorityLow",300);
	if(prefs->PriorityLow != 0){
		if(prefs->PriorityLow < prefs->PriorityHigh)
			prefs->PriorityLow = prefs->PriorityHigh*2;
	} else {
		prefs->PriorityLow = 300;
		prefs->PriorityHigh = 100;
	}
	prefs->RestartWaiting = ini.GetBool("RestartWaiting",true);
	prefs->LogToFile = ini.GetBool("LogToFile",false);
	prefs->TransferFullChunks = ini.GetBool("TransferFullChunks",true);
	prefs->BugReport = ini.GetBool("BugReport",false);
	prefs->DisableXSUpTo = ini.GetBool("DisableXSUpTo",false);
	prefs->XSUpTo=ini.GetInt("XSUpTo",GetMaxSourcePerFileSoft());
	if(prefs->XSUpTo > GetMaxSourcePerFileSoft())
		prefs->XSUpTo = GetMaxSourcePerFileSoft();
	sprintf(prefs->UsedFont,"%s",ini.GetString("UsedFont", "MS Sans Serif"));
	prefs->FontSize=ini.GetInt("FontSize",80);
	prefs->HashingPriority=ini.GetInt("HashingPriority",2);
	prefs->UploadParts = ini.GetBool("UploadParts",false);
	m_bA4AFStringEnabled.Put(ini.GetBool("A4AFStringEnabled", false));
	m_bBanMessageEnabled.Put(ini.GetBool("BanMessageEnabled", false));
	m_nSearchMethod.Put(ini.GetInt("SearchMethod",0));
	m_bBanMessageEnabled.Put(ini.GetBool("BanMessageEnabled", false));
	m_bAutoTakeLinks.Put(ini.GetBool("AutoTakeED2KLinks",false));
	m_nFileBufferSize.Put(ini.GetInt("FileBufferSizePref",16));
	m_nMainProcessPriority.Put(ini.GetInt("MainProcessPriority",0));

	m_bNSEnabled.Put(ini.GetBool("NSEnabled", false));
	m_bNSExceptSat.Put(ini.GetBool("NSExceptSat", true));
	m_bNSExceptSun.Put(ini.GetBool("NSExceptSun", true));
	m_nNSStart.Put(ini.GetInt("NSStart",64800));
	m_nNSEnd.Put(ini.GetInt("NSEnd",21600));
	m_nNSDayUpload.Put(ini.GetInt("NSDayUp",UNLIMITED));
	m_nNSDayDownload.Put(ini.GetInt("NSDayDown",UNLIMITED));
	m_nNSNightUpload.Put(ini.GetInt("NSNightUp",UNLIMITED));
	m_nNSNightDownload.Put(ini.GetInt("NSNightDown",UNLIMITED));

	// EpicStruggle
	prefs->m_multiple = ini.GetBool("MultipleInstances",false);

	ini.SerGet(true, prefs->downloadColumnWidths,
		ELEMENT_COUNT(prefs->downloadColumnWidths), "DownloadColumnWidths");
	ini.SerGet(true, prefs->downloadColumnHidden,
		ELEMENT_COUNT(prefs->downloadColumnHidden), "DownloadColumnHidden");
	ini.SerGet(true, prefs->downloadColumnOrder,
		ELEMENT_COUNT(prefs->downloadColumnOrder), "DownloadColumnOrder");
	ini.SerGet(true, prefs->uploadColumnWidths,
		ELEMENT_COUNT(prefs->uploadColumnWidths), "UploadColumnWidths");
	ini.SerGet(true, prefs->uploadColumnHidden,
		ELEMENT_COUNT(prefs->uploadColumnHidden), "UploadColumnHidden");
	ini.SerGet(true, prefs->uploadColumnOrder,
		ELEMENT_COUNT(prefs->uploadColumnOrder), "UploadColumnOrder");
	ini.SerGet(true, prefs->queueColumnWidths,
		ELEMENT_COUNT(prefs->queueColumnWidths), "QueueColumnWidths");
	ini.SerGet(true, prefs->queueColumnHidden,
		ELEMENT_COUNT(prefs->queueColumnHidden), "QueueColumnHidden");
	ini.SerGet(true, prefs->queueColumnOrder,
		ELEMENT_COUNT(prefs->queueColumnOrder), "QueueColumnOrder");
	ini.SerGet(true, prefs->searchColumnWidths,
		ELEMENT_COUNT(prefs->searchColumnWidths), "SearchColumnWidths");
	ini.SerGet(true, prefs->searchColumnHidden,
		ELEMENT_COUNT(prefs->searchColumnHidden), "SearchColumnHidden");
	ini.SerGet(true, prefs->searchColumnOrder,
		ELEMENT_COUNT(prefs->searchColumnOrder), "SearchColumnOrder");
	ini.SerGet(true, prefs->sharedColumnWidths,
		ELEMENT_COUNT(prefs->sharedColumnWidths), "SharedColumnWidths");
	ini.SerGet(true, prefs->sharedColumnHidden,
		ELEMENT_COUNT(prefs->sharedColumnHidden), "SharedColumnHidden");
	ini.SerGet(true, prefs->sharedColumnOrder,
		ELEMENT_COUNT(prefs->sharedColumnOrder), "SharedColumnOrder");
	ini.SerGet(true, prefs->serverColumnWidths,
		ELEMENT_COUNT(prefs->serverColumnWidths), "ServerColumnWidths");
	ini.SerGet(true, prefs->serverColumnHidden,
		ELEMENT_COUNT(prefs->serverColumnHidden), "ServerColumnHidden");
	ini.SerGet(true, prefs->serverColumnOrder,
		ELEMENT_COUNT(prefs->serverColumnOrder), "ServerColumnOrder");
	ini.SerGet(true, prefs->ircColumnWidths,
		ELEMENT_COUNT(prefs->ircColumnWidths), "IrcColumnWidths");
	ini.SerGet(true, prefs->ircColumnHidden,
		ELEMENT_COUNT(prefs->ircColumnHidden), "IrcColumnHidden");
	ini.SerGet(true, prefs->ircColumnOrder,
		ELEMENT_COUNT(prefs->ircColumnOrder), "IrcColumnOrder");

	// Barry - Provide a mechanism for all tables to store/retrieve sort order
	prefs->tableSortItemDownload = ini.GetInt("TableSortItemDownload", 0);
	prefs->tableSortItemUpload = ini.GetInt("TableSortItemUpload", 0);
	prefs->tableSortItemQueue = ini.GetInt("TableSortItemQueue", 0);
	prefs->tableSortItemSearch = ini.GetInt("TableSortItemSearch", 0);
	prefs->tableSortItemShared = ini.GetInt("TableSortItemShared", 0);
	prefs->tableSortItemServer = ini.GetInt("TableSortItemServer", 0);
	prefs->tableSortAscendingDownload = ini.GetBool("TableSortAscendingDownload", true);
	prefs->tableSortAscendingUpload = ini.GetBool("TableSortAscendingUpload", true);
	prefs->tableSortAscendingQueue = ini.GetBool("TableSortAscendingQueue", true);
	prefs->tableSortAscendingSearch = ini.GetBool("TableSortAscendingSearch", true);
	prefs->tableSortAscendingShared = ini.GetBool("TableSortAscendingShared", true);
	prefs->tableSortAscendingServer = ini.GetBool("TableSortAscendingServer", true);

	if (prefs->statsAverageMinutes<1) prefs->statsAverageMinutes=5;

	CString buffer2;
	for (int i=0;i<15;i++) {
		buffer2.Format("StatColor%i",i);
		sprintf(buffer,ini.GetString(buffer2,"-1"));	//we can now have black as background
		prefs->statcolors[i]=_atoi64(buffer);
		if (prefs->statcolors[i]==-1) {prefs->statcolors[i]=DefaultStatsColor(i);}
	}
	m_nGraphRatio.Put(ini.GetInt("ConnectionsGraphRatio",1)); // propstats Cax2

	// Added By Maverick
	prefs->MlDonkeyCM = ini.GetBool("MlDonkeyCM",true);

	// Added by Tarod -to load hash of A4AF auto file
	char hex_byte[3];
	int byte;
	CString A4AF_hash = ini.GetString("A4AFAutoFileHash", "00000000000000000000000000000000");
	if (A4AF_hash.GetLength() != 32) A4AF_hash = "00000000000000000000000000000000";
	hex_byte[2] = '\0';
	for (int i = 0, j = 0; i < 16; i++, j += 2) {
		hex_byte[0] = A4AF_hash.GetAt(j);
		hex_byte[1] = A4AF_hash.GetAt((j + 1));
		sscanf(hex_byte, "%02X", &byte);
		prefs->A4AF_file_hash[i] = (uchar)byte;
	}

	sprintf(buffer,ini.GetString("TotalDownloadedBytes",0,"Statistics"));
	m_nTotalDownloadedBytes.Put(_atoi64(buffer));

	sprintf(buffer,ini.GetString("TotalUploadedBytes",0,"Statistics"));
	m_nTotalUploadedBytes.Put(_atoi64(buffer));
	// Maverick
	prefs->proxy.EnablePassword = ini.GetBool("EnablePassword",false,"Proxy");
	prefs->proxy.UseProxy = ini.GetBool("EnableProxy",false,"Proxy");
	sprintf(buffer,"");
	sprintf(prefs->proxy.name,"%s",ini.GetString("Name",buffer,"Proxy"));
	sprintf(prefs->proxy.password,"%s",ini.GetString("Password",buffer,"Proxy"));
	sprintf(prefs->proxy.user,"%s",ini.GetString("User",buffer,"Proxy"));
	prefs->proxy.port = ini.GetInt("Port",1080,"Proxy");
	prefs->proxy.type = ini.GetInt("Type",PROXYTYPE_NOPROXY,"Proxy");
	SetLanguage();

	// Web Server
	m_sWebPassword.Put(ini.GetString("Password", "mypass", "WebServer").GetBuffer(0));
	m_nWebPort.Put(ini.GetInt("Port", 4711, "WebServer"));
	m_bWebEnabled.Put(ini.GetBool("Enabled", false, "WebServer"));
	m_bWebUseGzip.Put(ini.GetBool("UseGzip", true, "WebServer"));
	m_nWebPageRefresh.Put(ini.GetInt("PageRefreshTime", 120, "WebServer"));

	//////////////////////////////////////////////////////////////////
	// LANCAST (moosetea) - Read the settings

	// Get the Defualt Local IP address, using this clients hostname,
	// we will use this ip as the "default" Lancast ip, if no ip is found in the ini file.
	char localHost[255];
	gethostname(localHost, 255);						// Get this clients hostname
	struct hostent *host = gethostbyname(localHost);	// Get the host info

	// Get this clients default ip as a string
	char t_strdefaultip[17];
	strcpy(t_strdefaultip,inet_ntoa (*(struct in_addr*)*host->h_addr_list));

	// Get the ini file settings
	m_LancastEnabled.Put(ini.GetBool("Enabled",false,"Lancast"));
	m_UseDefaultIP.Put(ini.GetBool("UseDefaultIP",true,"Lancast"));
	m_DefaultLancastIP.Put(inet_addr(ini.GetString("IPAddress",t_strdefaultip,"Lancast")));
	m_LancastIP.Put(inet_addr(ini.GetString("IPAddress",t_strdefaultip,"Lancast")));
	m_LancastSubnet.Put(inet_addr(ini.GetString("Subnet","255.255.0.0","Lancast")));
	m_LancastPort.Put(ini.GetInt("Port",5000,"Lancast"));

	/////////////////////////////////////////////////////////////////////////////

	//START enkeyDEV(kei-kun) -Messaging preferences-
	m_nAcceptMessagesFrom.Put(ini.GetInt("AcceptMessageFrom",1));
	m_nAwayState.Put(false);
	m_sAwayStateMessage.Put(ini.GetString("AwayStateMessage","").GetBuffer(0));
	//END enkeyDEV(kei-kun) -Messaging preferences-
}

// LANCAST (moosetea) - get the lancast ip address
uint32 CPreferences::GetLancastIP(){
	if (GetLancastUseDefaultIP())
		return m_DefaultLancastIP.Get();
	else
		return m_LancastIP.Get();
}

WORD CPreferences::GetWindowsVersion(){
	static bWinVerAlreadyDetected = false;
	if(!bWinVerAlreadyDetected)
	{	
		bWinVerAlreadyDetected = true;
		m_wWinVer = DetectWinVersion();	
	}	
	return m_wWinVer;
}

uint16 CPreferences::GetMaxConperFive(){
	CSingleLock PrefLock(&m_Pref_Mutex_MaxConPer5,TRUE);
	return prefs->MaxConperFive;
}

uint16 CPreferences::GetDefaultMaxConperFive(){
	switch (GetWindowsVersion()){
		case _WINVER_98_:
			return 5;
		case _WINVER_95_:	
			return 5;
		case _WINVER_ME_:
			return 10;
		case _WINVER_2K_:
			return 20;
		case _WINVER_XP_:
			return 20;
		default:
			return 10;
	}
}

// Barry - Provide a mechanism for all tables to store/retrieve sort order
int CPreferences::GetColumnSortItem(Table t) const
{
	switch(t) 
	{
		case tableDownload:
			return prefs->tableSortItemDownload;
		case tableUpload:
			return prefs->tableSortItemUpload;
		case tableQueue:
			return prefs->tableSortItemQueue;
		case tableSearch:
			return prefs->tableSortItemSearch;
		case tableShared:
			return prefs->tableSortItemShared;
		case tableServer:
			return prefs->tableSortItemServer;
	}
	return 0;
}

// Barry - Provide a mechanism for all tables to store/retrieve sort order
bool CPreferences::GetColumnSortAscending(Table t) const
{
	switch(t) 
	{
		case tableDownload:
			return prefs->tableSortAscendingDownload;
		case tableUpload:
			return prefs->tableSortAscendingUpload;
		case tableQueue:
			return prefs->tableSortAscendingQueue;
		case tableSearch:
			return prefs->tableSortAscendingSearch;
		case tableShared:
			return prefs->tableSortAscendingShared;
		case tableServer:
			return prefs->tableSortAscendingServer;
	}
	return true;
}

// Barry - Provide a mechanism for all tables to store/retrieve sort order
void CPreferences::SetColumnSortItem(Table t, int sortItem)
{
	switch(t) 
	{
		case tableDownload:
			prefs->tableSortItemDownload = sortItem;
			break;
		case tableUpload:
			prefs->tableSortItemUpload = sortItem;
			break;
		case tableQueue:
			prefs->tableSortItemQueue = sortItem;
			break;
		case tableSearch:
			prefs->tableSortItemSearch = sortItem;
			break;
		case tableShared:
			prefs->tableSortItemShared = sortItem;
			break;
		case tableServer:
			prefs->tableSortItemServer = sortItem;
			break;
	}
}

// Barry - Provide a mechanism for all tables to store/retrieve sort order
void CPreferences::SetColumnSortAscending(Table t, bool sortAscending)
{
	switch(t) 
	{
		case tableDownload:
			prefs->tableSortAscendingDownload = sortAscending;
			break;
		case tableUpload:
			prefs->tableSortAscendingUpload = sortAscending;
			break;
		case tableQueue:
			prefs->tableSortAscendingQueue = sortAscending;
			break;
		case tableSearch:
			prefs->tableSortAscendingSearch = sortAscending;
			break;
		case tableShared:
			prefs->tableSortAscendingShared = sortAscending;
			break;
		case tableServer:
			prefs->tableSortAscendingServer = sortAscending;
			break;
	}
}

void CPreferences::SetLanguage() {
	switch(m_nLanguageID.Get()) {
		// read from ini and supported by the program?
		case MAKELANGID(LANG_ENGLISH,SUBLANG_DEFAULT):setlocale( LC_ALL, "english");break;
		case MAKELANGID(LANG_GERMAN,SUBLANG_DEFAULT):setlocale( LC_ALL, "german");break;
		case MAKELANGID(LANG_FRENCH,SUBLANG_DEFAULT):setlocale( LC_ALL, "french");break;
		case MAKELANGID(LANG_DANISH,SUBLANG_DEFAULT):setlocale( LC_ALL, "danish");break;
		case MAKELANGID(LANG_SPANISH,SUBLANG_SPANISH):setlocale( LC_ALL, "spanish");break;
		case MAKELANGID(LANG_DUTCH,SUBLANG_DEFAULT):setlocale( LC_ALL, "dutch");break;
		case MAKELANGID(LANG_RUSSIAN,SUBLANG_DEFAULT):setlocale( LC_ALL, "russian");break;
		case MAKELANGID(LANG_CHINESE,SUBLANG_CHINESE_SIMPLIFIED):setlocale( LC_ALL, "chs");break;
		case MAKELANGID(LANG_CHINESE,SUBLANG_CHINESE_TRADITIONAL):setlocale( LC_ALL, "cht");break;
		case MAKELANGID(LANG_KOREAN,SUBLANG_DEFAULT):setlocale( LC_ALL, "korean");break;
		case MAKELANGID(LANG_POLISH,SUBLANG_DEFAULT):setlocale( LC_ALL, "polish");break;
		case MAKELANGID(LANG_PORTUGUESE,SUBLANG_PORTUGUESE_BRAZILIAN):setlocale( LC_ALL, "ptb");break;
		case MAKELANGID(LANG_FINNISH,SUBLANG_DEFAULT):setlocale( LC_ALL, "finnish");break;
		case MAKELANGID(LANG_ITALIAN,SUBLANG_DEFAULT):setlocale( LC_ALL, "italian");break;
		case MAKELANGID(LANG_CATALAN,SUBLANG_DEFAULT):setlocale( LC_ALL, "esp");break;
		case MAKELANGID(LANG_GREEK,SUBLANG_DEFAULT):setlocale( LC_ALL, "greek");break;
		case MAKELANGID(LANG_HEBREW,SUBLANG_DEFAULT):setlocale( LC_ALL, "hebrew");break;
		case MAKELANGID(LANG_LITHUANIAN,SUBLANG_DEFAULT):setlocale( LC_ALL, "lithuanian");break;
		default:{
			uint8 nLocalLang = ::GetThreadLocale();
			
            CString strLng1 = GetResString(IDS_MB_LANGUAGEINFO,MAKELANGID(LANG_ENGLISH,SUBLANG_DEFAULT));
			CString strLng2 = "";
			switch(nLocalLang) {
				case	LANG_ENGLISH:
				case	LANG_GREEK:
				case 	LANG_GERMAN:
				case 	LANG_FRENCH:
				case 	LANG_DANISH:
				case 	LANG_SPANISH:
				case 	LANG_DUTCH:
				case 	LANG_RUSSIAN:
				case 	LANG_CHINESE:
				case 	LANG_KOREAN:
				case 	LANG_POLISH:
				case 	LANG_PORTUGUESE:
				case	LANG_FINNISH:
				case	LANG_ITALIAN:
				case	LANG_CATALAN:
				case	LANG_HEBREW:
				case	LANG_LITHUANIAN:
				{
					m_nLanguageID.Put(MAKELANGID(nLocalLang,SUBLANG_DEFAULT));
					SetThreadLocale(m_nLanguageID.Get());
					//strLng2 = GetResString(IDS_MB_LANGUAGEINFO,prefs->languageID);
					strLng1 = GetResString(IDS_MB_LANGUAGEINFO, m_nLanguageID.Get());//Cax2 fix
					break;
				}
				default:
					m_nLanguageID.Put(MAKELANGID(LANG_ENGLISH,SUBLANG_DEFAULT));
					SetThreadLocale(m_nLanguageID.Get());
			}

			CString strMessage = strLng1 + CString("\n\n") + strLng2;;
			MessageBox(0, strMessage.GetBuffer(),"Language",64);
			
			// DonGato
			CString strQuestion;
			if (nLocalLang == LANG_SPANISH)
				strQuestion = CString("\xbf") + GetResString(IDS_MULTIPLE, m_nLanguageID.Get()) + CString("?");
			else
				strQuestion = GetResString(IDS_MULTIPLE, m_nLanguageID.Get()) + CString("?");
			
			if (IDNO==MessageBox(0, strQuestion.GetBuffer(),"eMule Plus",MB_YESNO))
				return;
			else
				prefs->m_multiple = 1;
		}
	}
	SetThreadLocale(m_nLanguageID.Get());

	// if loading a string fails, set language to English
	if (GetResString(IDS_MB_LANGUAGEINFO, m_nLanguageID.Get()).GetLength()==0) {
		m_nLanguageID.Put(MAKELANGID(LANG_ENGLISH,SUBLANG_DEFAULT));
		SetThreadLocale(m_nLanguageID.Get());
	}
}

void CPreferences::SetMainProcessPriority(int Value) 			
{
	m_nMainProcessPriority.Put(Value);
}
